<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use App\Models\User;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Validator;

class AuthController extends Controller
{

	public function __construct(){}

	public function index()
    {
		$data['title'] = 'Iniciar sesión';
		return view('admin.auth',$data);
    }

    public function store(Request $request)
    {
		$validator = Validator::make($request->all(), [
            'email' => 'required|string|email|max:255|exists:users',
            'password' => 'required'
        ],[
			'email.required' => 'El correo es requerido',
			'email.string' => 'El correo debe ser un texto',
			'email.email' => 'El correo debe ser una dirección de correo electrónico válido',
            'email.max' => 'El correo no debe exceder más de 255 caracteres',
            'email.exists' => 'El correo no está registrado',
			'password.required' => 'La contraseña es requerida',
		]);
		if ($validator->fails()) {
			return redirect('admin/login')->withErrors($validator)->withInput();
		}
        $data = request()->except(['_token','_method']);
		if (!Auth::attempt($data)) {
			$request->session()->flash('error_attempt', 'Las credenciales no son correctas');
			return redirect('admin/login')->withErrors(['error_attempt' => 'Las credenciales no son correctas'])->withInput();
		}
		if(Auth::user()->role != 'Administrador'){
			Auth::logout();
			$request->session()->invalidate();
			$request->session()->regenerateToken();
			return redirect('/');
		}
		return redirect('admin');
    }

    public function logout(Request $request)
    {
        Auth::logout();
		$request->session()->invalidate();
		$request->session()->regenerateToken();
		return redirect('admin/login');
    }
}
