<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Auth;
use App\Models\User;

class ProfileController extends Controller
{
    private $tag_the = 'El';
    private $tag_o = 'o';
    private $r_name = 'admin.profile';
    private $c_name = 'Perfil';
    private $c_names = 'Perfil';
	private $list_tbl_fsc = ['name' => 'Nombre','username' => 'Usuario','email' => 'Correo','phone' => 'Tel&eacute;fono','role' => 'Rol'];
	private $o_model = User::class;
	
	
	/**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        $id = Auth::user()->id;
		$o = $this->o_model::find($id);
		$data['menu'] = 'admin/profile';
        $data['c_name'] = $this->c_name;
        $data['c_names'] = $this->c_names;
        $data['title'] = 'Mi perfil';
        $data['o'] = $o;
		return view($this->r_name.'.edit',$data);
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request)
    {
        $id = Auth::user()->id;
		$data = request()->except(['_token','_method']);
		$msj_success = 'El perfil ha sido actualizado correctamente.';
		if(!empty($data['passwordtoken'])){
			$validatedData = $request->validate([
				'oldpass' => 'required|string',
				'password' => 'required|string|min:6|confirmed',
				'password_confirmation' => 'required|same:password|min:6',
			],[
				'oldpass.required' => 'Se requiere contraseña anterior',
				'oldpass.min' => 'La contraseña anterior debe tener al menos 6 caracteres',
				'password.required' => 'Se requiere la nueva contraseña',
				'password.min' => 'La contraseña debe tener al menos 6 caracteres',
				'password_confirmation.required' => 'Las contraseñas no coinciden'
			]);
			if (!(Hash::check($request->get('oldpass'), Auth::user()->password))) {
				return redirect()->back()->with("msj_error","Su contraseña actual no coincide con la contraseña que proporcionó. Inténtalo de nuevo.");
			}
			$user = Auth::user();
			$user->password = Hash::make($request->get('password'));
			$user->save();
			$msj_success = 'Su contraseña ha sido actualizada correctamente.';
		} else {
			$validatedData = $request->validate([
				'name' => 'required|string|max:255',
				'email' => 'required|string|email|max:255|unique:users,email,'.$id,
			],[
				'name.required' => 'El nombre es requerido',
				'name.string' => 'El nombre debe ser un texto',
				'name.max' => 'El nombre no debe exceder más de 255 caracteres',
				'email.required' => 'El correo es requerido',
				'email.email' => 'El correo debe ser un correo válido',
				'email.string' => 'El correo debe ser un texto',
				'email.max' => 'El correo no debe exceder más de 255 caracteres',
				'email.unique' => 'El correo ya existe en la base de datos',
			]);
			$o = $this->o_model::find($id);
			if($request->has('photo')){
				$data['photo'] = $request->file('photo')->store('uploads','public');
			}
			$o->update($data);
		}
		$request->session()->flash('msj_success', $msj_success);
		return redirect('admin/profile');
    }
}
