<?php

namespace App\Http\Controllers;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use App\Models\User;
use App\Models\Plans;
use App\Models\Payments;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Validator;
use Laravel\Sanctum\PersonalAccessToken;
use App\Services\SupabaseService;
use Carbon\Carbon;
use App\Models\SupabaseAccessTokens;
use App\Models\Pages;
use App\Models\Subaccounts;
use Illuminate\Support\Str;
use hisorange\BrowserDetect\Parser as Browser;

class AuthController extends Controller
{
    //private $o_model = User::class;
	protected $supabaseService;
	public function __construct(SupabaseService $supabaseService){
	    $this->supabaseService = $supabaseService;
	}
	
	public function index()
    {
		setcookie('prestigio_session', '', time() - 3600, '/', '.prestigioapp.com');
        setcookie('prestigio_session', '', time() - 3600, '/', 'prestigioapp.com');
		$data['title'] = 'Iniciar sesión';
		return view('auth.index',$data);
    }

    public function store(Request $request)
    {
		$validator = Validator::make($request->all(), [
            'email' => 'required|string|email|max:255|exists:users',
            'password' => 'required'
        ],[
			'email.required' => 'El correo es requerido',
			'email.string' => 'El correo debe ser un texto',
			'email.email' => 'El correo debe ser una dirección de correo electrónico válido',
            'email.max' => 'El correo no debe exceder más de 255 caracteres',
            'email.exists' => 'El correo no está registrado',
			'password.required' => 'La contraseña es requerida',
		]);
		if ($validator->fails()) {
			return redirect('login')->withErrors($validator)->withInput();
		}
        $data = request()->except(['_token','_method']);
		if (!Auth::attempt($data)) {
			$request->session()->flash('error_attempt', 'Las credenciales no son correctas');
			return redirect('login')->withErrors(['error_attempt' => 'Las credenciales no son correctas'])->withInput();
		}
		//Limite de dispositivos
		$devices = 1;
		$user = User::where('email', $data['email'])->first();
		try {
            $response = $this->supabaseService->login($data['email'], $data['password']);

            if (isset($response['access_token'])) {
                // Almacenar el token en la sesión
                // Session::put('access_token', $response['access_token']);
                $expiresAt = Carbon::createFromTimestamp($response['expires_at'])->toDateTimeString();
                $supa_t = SupabaseAccessTokens::create([
                    'user_id' => $user->id,
                    'token' => $response['access_token'],
                    'expires_at' => $expiresAt,
                    'is_valid' => true,
                ]);
                session(['supabase_token' => $response['access_token']]);
            } else {
                $request->session()->flash('error_attempt', 'Las credenciales no son correctas');
                return redirect('login')->withErrors(['error_attempt' => 'Las credenciales no son correctas'])->withInput();
            }
        } catch (\Exception $e) {
            $request->session()->flash('error_attempt', 'Hubo un error en la autenticación');
            return redirect('login')->withErrors(['error_attempt' => 'Hubo un error en la autenticación'])->withInput();
        }
		//Verificar que el dispositivo del que desea ingresar sea uno que ya esta registrado
		$agent = request()->header('user-agent');
		$arr = ['agent' => $agent,'deviceType' => Browser::deviceType(),'platformName' => Browser::platformName(),'platformFamily' => Browser::platformFamily(),'deviceFamily' => Browser::deviceFamily(),'isInApp' => Browser::isInApp()];
		//Eliminados el acceso en caso de que exista
		$abilities = json_encode($arr);
		$issuedToken = PersonalAccessToken::where('abilities', $abilities)->where('tokenable_type', User::class)->where('tokenable_id', $user->id)->first();
		if(!empty($issuedToken->id)){
			PersonalAccessToken::destroy($issuedToken->id);
		}
		//Validamos que no exada el limite
		/*$o_lmb = $this->get_last_membership($user->id);
		if($o_lmb != FALSE){
			$devices = $o_lmb->devices;
		}
		$issuedTokens = PersonalAccessToken::where('tokenable_type', User::class)->where('tokenable_id', $user->id)->get();
		if ($issuedTokens->count() > $devices) {
			Auth::logout();
			$request->session()->flash('error_attempt', 'No puedes acceder a la plataforma porque has completado el limite('.$devices.') de dispositivos conectados');
			return redirect('login')->withErrors(['error_attempt' => 'No puedes acceder a la plataforma porque has completado el limite('.$devices.') de dispositivos conectados'])->withInput();
		}*/
		//
		$token = $user->createToken("auth_token",$arr);
		return redirect('/');
		
		
		/*$validator = Validator::make($request->all(), [
            'email' => 'required|string|email|max:255|exists:users',
            'password' => 'required'
        ],[
			'email.required' => 'El correo es requerido',
			'email.string' => 'El correo debe ser un texto',
			'email.email' => 'El correo debe ser una dirección de correo electrónico válido',
            'email.max' => 'El correo no debe exceder más de 255 caracteres',
            'email.exists' => 'El correo no está registrado',
			'password.required' => 'La contraseña es requerida',
		]);
		if ($validator->fails()) {
			return redirect('login')->withErrors($validator)->withInput();
		}
        $data = request()->except(['_token','_method']);
		if (!Auth::attempt($data)) {
			$request->session()->flash('error_attempt', 'Las credenciales no son correctas');
			return redirect('login')->withErrors(['error_attempt' => 'Las credenciales no son correctas'])->withInput();
		}
		//Limite de dispositivos
		$devices = 1;
		$user = User::where('email', $data['email'])->first();
		
		//Verificar que el dispositivo del que desea ingresar sea uno que ya esta registrado
		$agent = request()->header('user-agent');
		$arr = ['agent' => $agent,'deviceType' => Browser::deviceType(),'platformName' => Browser::platformName(),'platformFamily' => Browser::platformFamily(),'deviceFamily' => Browser::deviceFamily(),'isInApp' => Browser::isInApp()];
		//Eliminados el acceso en caso de que exista
		$abilities = json_encode($arr);
		$issuedToken = PersonalAccessToken::where('abilities', $abilities)->where('tokenable_type', User::class)->where('tokenable_id', $user->id)->first();
		if(!empty($issuedToken->id)){
			PersonalAccessToken::destroy($issuedToken->id);
		}
		//Validamos que no exada el limite
		$o_lmb = $this->get_last_membership($user->id);
		if($o_lmb != FALSE){
			$devices = $o_lmb->devices;
		}
		$issuedTokens = PersonalAccessToken::where('tokenable_type', User::class)->where('tokenable_id', $user->id)->get();*/
		/*
		if ($issuedTokens->count() > $devices) {
			Auth::logout();
			$request->session()->flash('error_attempt', 'No puedes acceder a la plataforma porque has completado el limite('.$devices.') de dispositivos conectados');
			return redirect('login')->withErrors(['error_attempt' => 'No puedes acceder a la plataforma porque has completado el limite('.$devices.') de dispositivos conectados'])->withInput();
		}
		*/
		//
		/*$token = $user->createToken("auth_token",$arr);
		$uid = mt_rand() . '_' . time().'.json';
		$path = "/home/movilprestigioap/public_html/tmp/$uid";
	    $data = array('email' => $_POST['email'],'password' => $_POST['password']);
	    file_put_contents($path,json_encode($data));

		return redirect('/?uid=' . $uid);*/
    }

    public function logout(Request $request)
    {
        $agent = request()->header('user-agent');
		$arr = ['agent' => $agent,'deviceType' => Browser::deviceType(),'platformName' => Browser::platformName(),'platformFamily' => Browser::platformFamily(),'deviceFamily' => Browser::deviceFamily(),'isInApp' => Browser::isInApp()];
		$abilities = json_encode($arr);
		$issuedTokens = PersonalAccessToken::where('abilities', $abilities)->where('tokenable_type', User::class)->where('tokenable_id', Auth::user()->id)->first();
		if(!empty($issuedTokens->id)){
			PersonalAccessToken::destroy($issuedTokens->id);
		}
		//$tokens = auth()->user()->tokens()->delete();
		Auth::logout();
		$request->session()->invalidate();
		$request->session()->regenerateToken();
		return redirect('/');
    }
	
	private function get_last_membership($id)
    {
		$o = Payments::where(['user' => $id])->orderBy('id', 'desc')->first();
		if(!empty($o->id)){
			//verificar que la fecha de expiration no este vencida
			$expiration = strtotime($o->expiration);
			$aux = date('Y-m-d');
			$currentDate = strtotime($aux);
			if($currentDate < $expiration){
				return Plans::where(['id' => $o->plan])->first();
			}
		}
		return FALSE;
    }
    public function isTokenExpired(Request $request)
    {
        $data = request()->except(['_token','_method']);
        $access_token = SupabaseAccessTokens::where('token', $data['token'])->first();
        if (isset($access_token) & $access_token->is_valid == 1) {
            try {
                $user = $this->supabaseService->getUser($access_token->token);
                if (isset($user['id'])) {
                    return response()->json([
                        'ok' => true
                    ]);
                }
                $access_token->update(['is_valid' => false]);
                return response()->json([
                    'ok' => false
                ]);
            } catch (\Throwable $th) {
                $access_token->update(['is_valid' => false]);
                return response()->json([
                    'ok' => false
                ]);

            }
        }
        return response()->json([
            'ok' => false
        ]);
    }

    private function last_membership($id)
    {
		$o = Payments::where(['user' => $id])->orderBy('id', 'desc')->first();
		if(!empty($o->id)){
			//verificar que la fecha de expiration no este vencida
			$expiration = strtotime($o->expiration);
			$aux = date('Y-m-d');
			$currentDate = strtotime($aux);
			if($currentDate < $expiration){
				return TRUE;
			}
		}
		return FALSE;
    }

    private function load_url($page,$code,$linkp = '',$user=null){
		//Paso 4, retornar la URL del nuevo acceso
		$o_page = Pages::where(['id' => $page])->first();
		$id_user = $user->id;
		if($user->role == 'subcuenta'){
			$o_sacc = Subaccounts::where(['id' => $user->sacc])->first();
			if(!empty($o_sacc->id)){
				$id_user = $o_sacc->user;
			}
		}
		$tag_url = 'url';
		if($this->last_membership($id_user)){
			$o_mb = Payments::where(['user' => $id_user])->orderBy('id', 'desc')->first();
			$o_plan = Plans::where(['id' => $o_mb->plan])->first();
			$tag_url = $o_plan->is_premium=='si'?'premium':'url';
		}
		$page_url = $o_page->$tag_url;
		$url = $page_url;
		$banner = '';
		if(!empty($linkp)){
			$banner = $page == 1?'/'.urlencode($linkp):'&lk='.urlencode($linkp);
		}
		if($page == 1){
			$url = $page_url;
		} else if($page == 2){
			$url = $page_url;
		} else if($page == 3){
			$url = $page_url;
		} else if($page == 4){
			$url = $page_url;
		} else if($page == 5){
			$url = $page_url;
		} else if($page == 6){
			$url = $page_url;
		} else if($page == 7){
			$url = $page_url;
		} else if($page == 8){
			//$url = $page_url.'register?code='.$code.'&p=4'.$banner;//https://prestigioapp.com/social/messages/
			$url = $o_page->$tag_url;
		} else if($page == 9){
			//$url = $page_url.'tempaccess?code='.$code.'&p=2'.$banner;
			$url = $page_url;
		} else if($page == 10){
			$url = $page_url;
			//$url = $page_url.'tempaccess?code='.$code.'&p=3'.$banner;
		}
		return $url;
	}
    //Load access
    private function log_sh_access($id,$user=null)
    {
        $path = 'https://prestigioapp.com/api/api/index';
		$o_page = Pages::where(['id' => $id])->first();
		if(empty($o_page->id)){
			return ['ok' => FALSE,'msj' => 'La pagina solicitada no existe'];
		}
		$code = Str::random(8);
		$o_user = $user;
		$id_user = $user->id;
		if($user->role == 'subcuenta'){
			$o_sacc = Subaccounts::where(['id' => $user->sacc])->first();
			if(!empty($o_sacc->id)){
				$id_user = $o_sacc->user;
				$o_user = User::where(['id' => $id_user])->first();
			}
		}
		return ['ok' => TRUE,'msj' => $code];
    }
	public function siteaccess(Request $request)
    {
		$data = request()->except(['_token','_method']);
		$user = !isset($data['email']) ? Auth::user() : User::where(['email' => $data['email']])->first();
		$res = $this->log_sh_access($data['page'],$user);
		if($res['ok']){
			$code = $res['msj'];
			$url = $this->load_url($data['page'],$code,'',$user);
            $token = session('supabase_token');
            return response()->json(
                [
                    "error"=> false,
                    "url" => $url,
                    "email" => $user->email,
                    "name" => $user->name,
                    "password" => $user->password,
                    "token" => $token
                ]);
		} else {
			echo $res['msj'];
		}
    }
}
