<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\Models\Payments;
use Illuminate\Support\Facades\Auth;

class MembershipController extends Controller
{
    /**
     *
     *  @OA\Get(
     *  path="/api/membership/all",
     *  summary="Lista de suscripciones",
     *  operationId="getAllMembership",
     *  tags={"Membership"},
     *  security={{ "apiAuth": {} }},
     *  @OA\Response(response=200, description="Obtiene la lista de suscripciones", @OA\JsonContent())
     * )
     **/
    public function index()
    {
        return $this->sendResponse(Payments::with(['user','plan'])->get(), 'Lista de suscripciones');
    }
	
    /**
     *
     *  @OA\Get(
     *  path="/api/membership",
     *  summary="Lista de mis suscripciones",
     *  operationId="getAllMyMembership",
     *  tags={"Membership"},
     *  security={{ "apiAuth": {} }},
     *  @OA\Response(response=200, description="Obtiene la lista de mis suscripciones", @OA\JsonContent())
     * )
     **/
    public function mymemb()
    {
        return $this->sendResponse(Payments::with(['user','plan'])->where(['user' => Auth::user()->id])->orderBy('id', 'desc')->get(), 'Mis suscripciones');
    }
	
    /**
     *
     *  @OA\Get(
     *  path="/api/membership/lastmemb",
     *  summary="Mi última suscripción",
     *  operationId="getAllMyLast",
     *  tags={"Membership"},
     *  security={{ "apiAuth": {} }},
     *  @OA\Response(response=200, description="Obtiene mi última suscripción", @OA\JsonContent())
     * )
     **/
    public function lastmemb()
    {
        return $this->sendResponse(Payments::with(['user','plan'])->where(['user' => Auth::user()->id])->orderBy('id', 'desc')->first(), 'Mi última suscripción');
    }
}
