<?php

namespace App\Http\Controllers;

use App\Http\Requests\UserRequest;
use App\Models\User;
use App\Models\Plans;
use App\Models\Payments;
use Illuminate\Support\Str;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Validator;
use Illuminate\Support\Facades\Mail;
use App\Mail\Ntfs;

class RegisterController extends Controller
{

    public function index()
    {
		if (!request()->session()->has('validkeyfsc')) {
			return redirect('register/vdemail');
		}
		$usemail = session('emailaccfsc');
		$data['title'] = 'Crear una cuenta';
		$data['usemail'] = $usemail;
		return view('auth.register',$data);
    }
	
	public function vdemail()
    {
		$data['title'] = 'Validar correo';
		return view('auth.vdemail',$data);
    }
	
	public function validaccount(Request $request)
    {
        $data = request()->except(['_token','_method']);
		$validator = Validator::make($data, [
			'email' => 'required|string|email|max:255|unique:users,email',
        ],[
			'email.required' => 'El correo es requerido',
            'email.email' => 'El correo debe ser un correo válido',
            'email.string' => 'El correo debe ser un texto',
            'email.max' => 'El correo no debe exceder más de 255 caracteres',
            'email.unique' => 'El correo ya existe en la base de datos',
		]);
		if ($validator->fails()) {
			return redirect('register/vdemail')->withErrors($validator)->withInput();
		}
        //$email = $data['email'];
		$key = $this->search_key();
		session(['validaccfsc' => $key]);
		session(['emailaccfsc' => $data['email']]);
		Mail::to($data['email'])->send(new Ntfs('Validad correo','Para continuar con el registro debes validar el correo, ingrese el siguiente código: '.$key,' ',$data['email']));
		
		return redirect('register/verificar');
    }

    public function verificar()
    {
		$data['title'] = 'Validar código';
		return view('auth.verificar',$data);
    }
	
	public function validcode(Request $request)
    {
        $data = request()->except(['_token','_method']);
		$validator = Validator::make($data, [
			'code' => 'required',
        ],[
			'code.required' => 'El código es requerido',
		]);
		if ($validator->fails()) {
			return redirect('register/verificar')->withErrors($validator)->withInput();
		}
		$key = '';
        if (request()->session()->has('validaccfsc')) {
			$key = session('validaccfsc');
		} else {
			return redirect('register/vdemail');
		}
		if($key != $data['code']){
			return redirect('register/verificar')->withErrors(['code' => 'El código es incorrecto'])->withInput();
		}
		session(['validkeyfsc' => $data['code']]);
		return redirect('register');
    }
	
    public function store(UserRequest $request)
    {
        $validator = Validator::make($request->all(), [
            'name' => 'required',
            'email' => 'required|string|email|max:255|unique:users',
            'password' => 'required'
        ],[
			'name.required' => 'El nombre es requerido',
			'email.required' => 'El correo es requerido',
			'email.string' => 'El correo debe ser un texto',
			'email.email' => 'El correo debe ser una dirección de correo electrónico válido',
            'email.max' => 'El correo no debe exceder más de 255 caracteres',
            'email.unique' => 'El correo está registrado en una cuenta',
			'password.required' => 'La contraseña es requerida',
		]);
		if ($validator->fails()) {
			return redirect('register')->withErrors($validator)->withInput();
		}
		$data = $request->except(['_token','_method']);
		$data['passwd'] = $data['password'];
        $data['password'] = bcrypt($data['password']);
        $o = User::create($data);
		$this->freeplan($o->id);
		Mail::to($o->email)->send(new Ntfs('Nueva cuenta','Enhorabuena!!! Bienvenid@ a Prestigio, su nueva cuenta ha sido registrada correctamente.',$o->name,$o->email));
        return redirect('/');
    }
	
	private function freeplan($user_id)
    {
        $o_plan = Plans::where(['is_init' => 'si','status' => 'active'])->first();
		if(empty($o_plan->id)){
			return false;
		}
		$x_id_invoice = uniqid();
		$expiration = $this->expiration($o_plan->month);
		$x_ref_payco = uniqid();
		$o = Payments::create(['user' => $user_id,'plan' => $o_plan->id,'invoice' => $x_id_invoice,'amount' => 0,'currency' => 'COP','response' => 'Aceptado','franchise' => 'Free','date_init' => date('Y-m-d'),'expiration' => $expiration,'description' => $x_ref_payco]);
		return true;
    }
	
	private function expiration($d = 30) {
		$fecha = date('Y-m-j');
		$nuevafecha = strtotime ( '+'.$d.' day' , strtotime ( $fecha ) ) ;
		$nuevafecha = date ( 'Y-m-d' , $nuevafecha );
		return $nuevafecha;
    }
	
	private function search_key($len = 8){
		$r = Str::random($len);
		return $r;
	}
}
