<?php

namespace App\Http\Controllers;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use Illuminate\Support\Str;
use App\Models\Subaccounts;
use App\Models\Subaccspage;
use App\Models\Pages;
use App\Models\User;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Validator;
use Illuminate\Support\Facades\Mail;
use Illuminate\Support\Facades\Hash;
use App\Mail\Ntfs;

class SubaccountsController extends Controller
{
	private $tag_the = 'La';
    private $tag_o = 'a';
    private $r_name = 'subaccounts';
    private $v_name = 'subaccounts';
    private $c_name = 'Sub cuenta';
    private $c_names = 'Sub cuentas';
	private $list_tbl_fsc = ['name' => 'Nombre','email' => 'Correo'];
	private $o_model = Subaccounts::class;

	public function __construct(){
    }

	/**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        $data['menu'] = $this->r_name;
        $data['tag_o'] = $this->tag_o;
        $data['tag_the'] = $this->tag_the;
        $data['c_name'] = $this->c_name;
        $data['c_names'] = $this->c_names;
        $data['list_tbl_fsc'] = $this->list_tbl_fsc;
        $data['title'] = 'Lista de '.$this->c_names;
		$data['o_all'] = $this->o_model::where(['user' => Auth::user()->id])->get();
		return view($this->v_name.'.index',$data);
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        $data['menu'] = $this->r_name;
        $data['c_name'] = $this->c_name;
        $data['c_names'] = $this->c_names;
        $data['title'] = 'Agregar '.$this->c_name;
		return view($this->v_name.'.create',$data);
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $data = request()->except(['_token','_method']);
		$validatedData = $request->validate([
			'name' => 'required|string',
			'email' => 'required|string|email|max:255|unique:users',
		],[
			'name.required' => 'El nombre es requerido',
			'name.string' => 'El nombre debe ser un texto',
			'email.required' => 'El correo es requerido',
            'email.email' => 'El correo debe ser un correo válido',
            'email.string' => 'El correo debe ser un texto',
            'email.max' => 'El correo no debe exceder más de 255 caracteres',
            'email.unique' => 'El correo ya esta registrado en una cuenta',
		]);
		$data['uuid'] = (string) Str::uuid();
		$data['user'] = Auth::user()->id;
		$o = $this->o_model::create($data);
		$password = Str::random(8);
		$o_us = User::create(['name' => $data['name'],'email' => $data['email'],'password' => Hash::make($password),'role' => 'subcuenta','sacc' => $o->id]);
		Mail::to($o_us->email)->send(new Ntfs('Nueva cuenta','Enhorabuena!!! El usuario '.Auth::user()->name.' le ha enviado una invitación a tener una sub-cuenta, para acceder debe ingresar con su correo y la siguiente contraseña: <b>'.$password.'</b>',$o_us->name,$o_us->email));
		$request->session()->flash('msj_success', $this->tag_the.' '.$this->c_name.' '.$request->name.' ha sido registrad'.$this->tag_o.' correctamente.');
		return redirect($this->r_name);
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        if(empty($id)){
			return redirect($this->r_name);
		}
		$tag = is_numeric($id)?'id':'uuid';
		$o = $this->o_model::where([$tag => $id])->first();
		if(empty($o->id)){
			return redirect($this->r_name);
		}
		$data['menu'] = $this->r_name;
        $data['c_name'] = $this->c_name;
        $data['c_names'] = $this->c_names;
        $data['title'] = 'Actualizar '.$this->c_name;
        $data['o'] = $o;
		$data['o_all'] = Pages::all();
		$i_all = Subaccspage::where(['subaccounts_id' => $o->id])->get();
		$list_i_all = [];
		if($i_all->count() > 0){
			foreach($i_all as $key => $row){
				$list_i_all[] = $row->pages_id;
			}
		}
		$data['i_all'] = $list_i_all;
		return view($this->v_name.'.edit',$data);
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id)
    {
        $data = request()->except(['_token','_method']);
		$validatedData = $request->validate([
			'name' => 'required|string',
		],[
			'name.required' => 'La pregunta es requerida',
		]);
		if(empty($id)){
			return redirect($this->r_name);
		}
		$tag = is_numeric($id)?'id':'uuid';
		$o = $this->o_model::where([$tag => $id])->first();
		if(empty($o->id)){
			return redirect($this->r_name);
		}
		$o->update($data);
		$request->session()->flash('msj_success', $this->tag_the.' '.$this->c_name.' '.$o->name.' ha sido actualizad'.$this->tag_o.' correctamente.');
		return redirect($this->r_name);
    }
	
	public function saccpages()
    {
        $id = $_POST['plan'];//Subcuenta
		$o = $this->o_model::where(['uuid' => $id])->first();
		if(empty($o->id)){
			return false;
		}
		$idpm = $_POST['id'];//Pagina
		$s = (string) $_POST['state'];
		if($s == 'true'){
			$o->pages()->attach($idpm);
		} else {
			$o->pages()->detach($idpm);
		}
		echo 'ok';
    }
}
