<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\Models\Tempaccess;//user,code
use App\Models\Userspages;//user,page,user_local
use App\Models\User;
use App\Models\Pages;
use App\Models\Payments;
use Illuminate\Support\Str;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Http;

class TempaccessController extends Controller
{

    /**
     *
     * @OA\Post(
     * path="/api/tempaccess",
     * summary="Acceder a una pagina",
     * operationId="newTempaccess",
     * tags={"Tempaccess"},
     * security={{ "apiAuth": {} }},
     *     	@OA\RequestBody(
     *          required=true,
     *          @OA\MediaType(
     *              mediaType="multipart/form-data",
     *              @OA\Schema(
     *                  required={"page"},
     *                  @OA\Property(property="page", type="integer", example="1"),
     *               ),
     *           ),
     *       ),
     * @OA\Response(response=200, description="response url", @OA\JsonContent()),
     * @OA\Response(response=400, description="Bad request", @OA\JsonContent()),
     * @OA\Response(response=404, description="Resource Not Found", @OA\JsonContent()),
     * )
     **/
    public function index(Request $request)
    {
        //$path = 'http://127.0.0.1/prestigioapi/api/index';
        //$path = 'https://api.prestigioapp.com/api/index';
        $path = 'https://prestigioapp.com/api/api/index';
		$data = $request->all();
		$o_page = Pages::where(['id' => $data['page']])->first();
		if(empty($o_page->id)){
			return $this->sendError("La pagina solicitada no existe",$data['page'],422);
		}
		$id_user = Auth::user()->id;
		//Paso 1, verificar que tenga suscripcion activa, sino tiene retornar ERROR.
		if(!$this->last_membership($id_user)){
			return $this->sendError("No tiene una suscripción activa",[],422);
		}
		//Paso 2, verificar que el usuario $id_user este registrado para la pagina $data['page'], sino esta registrado registrarlo.
		$o_up = Userspages::where(['user' => $id_user,'page' => $data['page']])->first();
		//$o_user = User::where(['id' => $id_user])->first();
		if(empty($o_up->id)){
			//Registrar al usuario en la pagina y obtener el ID de ese registro
			if($data['page'] == 4){
				$path = 'http://co.prestigioapp.com/newaccount';
			}
			$user_local = $this->sendpost($path, [
				'acc' => 'register',
				'page' => $data['page'],
				'name' => Auth::user()->name,
				'email' => Auth::user()->email,
				'api_token' => Str::random(40)
			]);
			if($user_local == false OR $user_local == 'false'){
				return $this->sendError("No se ha podido completar el registro",$user_local,422);
			}
			$o_up = Userspages::create(['user' => $id_user,'page' => $data['page'],'user_local' => $user_local]);
		}
		//Paso 3, generar un enlace de acceso para la pagina $data['page'] solicitada
		$code = Str::random(8);
		if($data['page'] == 4){
			$o_new = Tempaccess::create(['user' => $o_up->user_local,'code' => $code]);
		} else {
			$response = $this->sendpost($path, [
				'acc' => 'access',
				'page' => $data['page'],
				'user' => $o_up->user_local,
				'code' => $code
			]);
		}
		//Paso 4, retornar la URL del nuevo acceso
        $url = $o_page->url.'tempaccess/'.$code;
		if($data['page'] == 2){
			$url = $o_page->url.'register?code='.$code;
		} else if($data['page'] == 3){
			$url = $o_page->url.'tempaccess?code='.$code;
		}
        return $this->sendResponse($url, "URL de la pagina");
    }

    /**
     *
     * @OA\Post(
     * path="/api/tempaccess/verykey",
     * summary="Verificar acceso",
     * operationId="newTempaccessKey",
     * tags={"Tempaccess"},
     * security={{ "": {} }},
     *     	@OA\RequestBody(
     *          required=true,
     *          @OA\MediaType(
     *              mediaType="multipart/form-data",
     *              @OA\Schema(
     *                  required={"code"},
     *                  @OA\Property(property="code", type="string", example="435evd34"),
     *               ),
     *           ),
     *       ),
     * @OA\Response(response=200, description="response user ID", @OA\JsonContent()),
     * @OA\Response(response=400, description="Bad request", @OA\JsonContent()),
     * @OA\Response(response=404, description="Resource Not Found", @OA\JsonContent()),
     * )
     **/
    public function verykey(Request $request)
    {
		$data = $request->all();
		$o = Tempaccess::where(['code' => $data['code']])->first();
		if(empty($o->id)){
			//return false;
			return response()->json(false, 400);
		}
		$out = $o->user;
		Tempaccess::destroy($o->id);
        //return $out;
		return response()->json($out, 200);
    }

    private function last_membership($id)
    {
		$o = Payments::where(['user' => $id])->orderBy('id', 'desc')->first();
		if(empty($o->id)){
			return FALSE;
		}
		//verificar que la fecha de expiration no este vencida
		$expiration = strtotime($o->expiration);
		$aux = date('Y-m-d');
		$currentDate = strtotime($aux);
		if($currentDate < $expiration){
			return TRUE;
		}
		return FALSE;
    }

    private function sendpost($url,$fields)
    {
		$fields_string = http_build_query($fields);
		$ch = curl_init();
		curl_setopt($ch,CURLOPT_URL, $url);
		curl_setopt($ch,CURLOPT_POST, true);
		curl_setopt($ch,CURLOPT_POSTFIELDS, $fields_string);
		curl_setopt($ch,CURLOPT_RETURNTRANSFER, true); 
		$result = curl_exec($ch);
		return $result;
    }
}
