<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\Models\User;
use Illuminate\Support\Str;
class UsersController extends Controller
{
	/**
     * Get users.
     *
     *  @OA\Get(
     *  path="/api/users",
     *  summary="Lista de usuarios",
     *  operationId="getAllusers",
     *  tags={"Users"},
     *  @OA\Response(response=200, description="Obtiene la lista de usuarios", @OA\JsonContent()),
     *  security={{ "apiAuth": {} }}
     * )
     **/
    public function index()
    {
        return $this->sendResponse(User::all(), 'Lista de usuarios');
    }
	
	/**
     *
     * @OA\Post(
     * path="/api/users",
     * summary="Crear nuevo usuario",
     * operationId="newUsers",
     * tags={"Users"},
     * security={{ "apiAuth": {} }},
     *     	@OA\RequestBody(
     *          required=true,
     *          @OA\MediaType(
     *              mediaType="multipart/form-data",
     *              @OA\Schema(
     *                  required={"name","email","password"},
     *                  @OA\Property(property="name", type="string", example="Jhon Doe"),
     *                  @OA\Property(property="email", type="string", example="jhondoe@fsc.com"),
     *                  @OA\Property(property="password", type="string", example="123456"),
     *               ),
     *           ),
     *       ),
     * @OA\Response(response=200, description="response create new item", @OA\JsonContent()),
     * @OA\Response(response=400, description="Bad request", @OA\JsonContent()),
     * @OA\Response(response=404, description="Resource Not Found", @OA\JsonContent()),
     * )
     **/
    public function store(Request $request)
    {
        $data = $request->all();//
        $data['password'] = bcrypt($data['password']);
        $o = User::create($data);
        return $this->sendResponse($o, 'El usuario fue creado exitosamente.');
    }
	
	/**
     *
     *  @OA\Get(
     *  path="/api/users/{id}",
     *  @OA\Parameter(description="ID del usuario", in="path",name="id", required=true,
     *     @OA\Schema(
     *       type="integer",
     *       format="int64",
     *       example="1"
     *     )
     *  ),
     *  summary="show usuario",
     *  operationId="getUsers",
     *  tags={"Users"},
     *  @OA\Response(response=200, description="mostrar detalles", @OA\JsonContent()),
     *  security={{ "apiAuth": {} }}
     * )
     **/
    public function show($id)
    {
        $o = User::where(['id' => $id])->first();
        return $this->sendResponse($o, "Detalles de usuario");
    }

    /**
     *
     * @OA\Post(
     * path="/api/users/update/{id}",
     * @OA\Parameter(description="ID del usuario", in="path",name="id", required=true,
     *     @OA\Schema(
     *       type="integer",
     *       format="int64",
     *       example="1"
     *     )
     * ),
     * summary="Actualizar usuario",
     * operationId="updateUsers",
     * tags={"Users"},
     * security={{ "apiAuth": {} }},
     *     	@OA\RequestBody(
     *          required=true,
     *          @OA\MediaType(
     *              mediaType="multipart/form-data",
     *              @OA\Schema(
     *                  required={"name","email","password"},
     *                  @OA\Property(property="name", type="string", example="Jhon Doe"),
     *                  @OA\Property(property="email", type="string", example="jhondoe@fsc.com"),
     *                  @OA\Property(property="password", type="string", example="123456"),
     *               ),
     *           ),
     *       ),
     * @OA\Response(response=200, description="response updated", @OA\JsonContent()),
     * @OA\Response(response=400, description="Bad request", @OA\JsonContent()),
     * @OA\Response(response=404, description="Resource Not Found", @OA\JsonContent()),
     * )
     **/
    public function update(Request $request, $id)
    {
        $data = $request->all();
        $o = User::where(['id' => $id])->first();
		$data['password'] = bcrypt($data['password']);
        $o->update($data);
        return $this->sendResponse($o, "El usuario ha sido actualizado correctamente");
    }

    /**
     *
     *  @OA\Delete(
     *  path="/api/users/{id}",
     * @OA\Parameter(description="ID del usuario", in="path",name="id", required=true,
     *     @OA\Schema(
     *       type="integer",
     *       format="int64",
     *       example="1"
     *     )
     * ),
     *  summary="delete item",
     *  operationId="DeleteUsers",
     *  tags={"Users"},
     *  @OA\Response(response=200, description="delete item", @OA\JsonContent()),
     *  security={{ "apiAuth": {} }}
     * )
     **/
    public function destroy($id)
    {
        $o = User::where(['id' => $id])->first();
        User::destroy($id);
        return $this->sendResponse($o, "El usuario ha sido eliminado correctamente");
    }
}
