<?php
// app/Http/Middleware/VerifySupabaseToken.php

namespace App\Http\Middleware;

use App\Models\SupabaseAccessTokens;
use App\Models\User;
use Closure;
use Illuminate\Http\Request;
use App\Services\SupabaseService;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Str;
class VerifySupabaseToken
{
    protected $supabaseService;

    public function __construct(SupabaseService $supabaseService)
    {
        $this->supabaseService = $supabaseService;
    }

    /**
     * The URIs that should be excluded from CSRF verification.
     *
     * @var array
     */
    protected $except = [
        'login',
        'user/verify_token',
        'siteaccess'
        //
    ];
    /**
     * Handle an incoming request.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \Closure  $next
     * @return mixed
     */
    public function handle(Request $request, Closure $next)
    {
        // $token = $request->header('Authorization') ?: $request->query('token');
        $token = $token = session('supabase_token');

        if (!$token) {
            if($request->isJson()){
                return response()->json(
                    [
                        "url" => "/login",
                        "email" => "",
                        "password" => "",
                        "token" => "",
                        "error" => true
                    ]);
            }
            return redirect('/login')->with('error', 'Unauthorized');
        }

        $token = str_replace('Bearer ', '', $token);

        try {
            $user = $this->supabaseService->getUser($token);
            if (isset($user['id'])) {
                return $next($request);
            }
            session()->forget('supabase_token');
            SupabaseAccessTokens::where('token', $token)->update(['is_valid' => false]);
            Auth::logout();
            return redirect('/login')->with('error', 'Invalid token');
        } catch (\Throwable $th) {
            session()->forget('supabase_token');
            SupabaseAccessTokens::where('token', $token)->update(['is_valid' => false]);
            Auth::logout();
            if($request->isJson()){
                return response()->json(
                    [
                        "url" => "/login",
                        "email" => "",
                        "password" => "",
                        "token" => "",
                        "error" => true
                    ]);
            }
            return redirect('/login')->with('error', 'Token expired');

        }
    }

}

