<?php

namespace App\Http\Requests;

use Illuminate\Foundation\Http\FormRequest;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Request;
use Illuminate\Validation\Rule;

class UserRequest extends FormRequest
{
    /**
     * Determine if the user is authorized to make this request.
     *
     * @return bool
     */
    public function authorize()
    {
        return true;
    }

    protected function failedValidation(\Illuminate\Contracts\Validation\Validator $validator)
    {
        $response = new JsonResponse([
            'data' => ['errors' => $validator->errors()],
            'message' => 'Los datos proporcionados no son válidos',
        ], 422);

        throw new \Illuminate\Validation\ValidationException($validator, $response);
    }

    /**
     * Get the validation rules that apply to the request.
     *
     * @return array
     */
    public function rules(Request $request)
    {
        $rule_unique = Rule::unique('users', 'email')->ignore($this->email);

        if ($request->isMethod('PUT')) {
            $rule_unique = Rule::unique('users', 'email')->ignore($this->email, 'email');
        }

        return [
            'name' => 'required',
            'email' => [
                'required',
                'email',
                $rule_unique
            ],
            'password' => 'required'
        ];
    }

    /**
     * Get the error messages for the defined validation rules.
     *
     * @return array
     */
    public function messages()
    {
        return [
            'name.required' => 'El nombre es obligatorio',
            'email.required' => 'El email es obligatorio',
            'email.unique' => 'El email es repetido',
            'email.email' => 'El email no es valido',
            'password.required' => 'El password es requerido'
        ];
    }
}
